__version__ = '1.0'
__author__ = 'Jonathan Camenzind (Jonathan.Camenzind@ch.gudel.com)'


from vcCommand import *


def SetGroup(arg):
  if groupNameProp.Value == '':
    app.messageBox(
      'Specify a group name first',
      'Empty group name',
      VC_MESSAGE_TYPE_INFO,
      VC_MESSAGE_BUTTONS_OK)
    return
  
  features = selectionManager.getSelection(VC_SELECTION_FEATURE)
  
  if len(features) == 0:
    app.messageBox(
      'Select the features you want to create a color group for',
      'No features selcted',
      VC_MESSAGE_TYPE_INFO,
      VC_MESSAGE_BUTTONS_OK)
    return
  
  comp = GetSelectedComponent()
  
  propName = propPrefix + groupNameProp.Value
  colorGroupProp = comp.getProperty(propName)
  
  if colorGroupProp is not None:
    result = app.messageBox(
      'A color group with this name already exists.\nWould you like to overwrite it?',
      'Color group already exists',
      VC_MESSAGE_TYPE_INFO,
      VC_MESSAGE_BUTTONS_YESNO)
    
    if result == VC_MESSAGE_RESULT_NO:
      return
    
    SafeDeleteProperty(comp, propName)
    SafeDeleteProperty(comp, propName + '_')
  
  prop = comp.createProperty('Ref<Material>', propName)
  prop.Value = None
  prop.IsVisible = True
  prop.OnChanged = OnColorChanged
  
  featuresProp = comp.createProperty(VC_STRING, propName + '_')
  featuresProp.Value = ','.join([x.Name for x in features])
  featuresProp.IsVisible = False
  
  UpdateColorsScript(comp)
  UpdateUI()

def RemoveGroup(arg):
  groupName = colorGroupsProp.Value
  
  propName = propPrefix + groupName
  
  result = app.messageBox(
    'Delete color group "' + groupName + '"?',
    'Delete color group?',
    VC_MESSAGE_TYPE_QUESTION,
    VC_MESSAGE_BUTTONS_YESNO)
  
  if result == VC_MESSAGE_RESULT_NO:
    return
  
  comp = GetSelectedComponent()
  
  SafeDeleteProperty(comp, propName)
  SafeDeleteProperty(comp, propName + '_')
  
  UpdateUI()

def SelectGroupFeatures(arg):
  groupName = colorGroupsProp.Value
  comp = GetSelectedComponent()
  
  featureIds = comp.getProperty(propPrefix + groupName + '_').Value.split(',')
  features = []
  
  for featureId in featureIds:
    features.append(comp.findFeature(featureId))
  
  selectionManager.setSelection(features)

def UpdateColorsScript(comp):
  script = comp.getBehaviour(scriptName)
  
  if script is None:
    script = comp.createBehaviour(VC_SCRIPT, scriptName)
  
  script.Script = ''
  script.Script = updateColorsScript

def UpdateUI():
  global suppressColorChange
  
  comp = GetSelectedComponent()
  
  if comp is None:
    SetUIVisibility(False)
    return
  
  colorGroupProps = [x for x in comp.Properties
    if x.Name.startswith(propPrefix) and x.Name[-1] != '_']
  
  if len(colorGroupProps) == 0:
    SetUIVisibility(False)
    return
  
  colorGroupsProp.StepValues = [x.Name[len(propPrefix):] for x in colorGroupProps]
  
  if colorGroupsProp.Value not in colorGroupsProp.StepValues:
    colorGroupsProp.Value = colorGroupsProp.StepValues[0]
  
  color = next((x for x in colorGroupProps
    if x.Name == propPrefix + colorGroupsProp.Value)).Value
  
  if color != colorProp.Value:
    suppressColorChange = True
    colorProp.Value = color
  
  SetUIVisibility(True)

def SetUIVisibility(setVisible):
  colorGroupsProp.IsVisible = setVisible
  colorProp.IsVisible = setVisible
  removeGroupProp.IsVisible = setVisible
  selectFeaturesProp.IsVisible = setVisible

def GetSelectedComponent():
  comps = selectionManager.getSelection(VC_SELECTION_COMPONENT)
  
  if len(comps) > 0:
    return comps[0]
  
  nodes = selectionManager.getSelection(VC_SELECTION_PARENTNODE)
  
  if len(nodes) == 0:
    return None
  
  return nodes[0].Component

def SafeDeleteProperty(comp, propName):
  prop = comp.getProperty(propName)
  
  if prop is not None:
    comp.deleteProperty(prop)

def OnColorGroupChanged(arg):
  UpdateUI()

def OnColorChanged(arg):
  global suppressColorChange
  
  if suppressColorChange:
    suppressColorChange = False
    return
  
  groupName = colorGroupsProp.Value
  comp = GetSelectedComponent()
  
  comp.getProperty(propPrefix + groupName).Value = arg.Value
  
  UpdateUI()

def OnStart():
  global groupNameProp, setGroupProp, colorGroupsProp
  global colorProp, removeGroupProp, selectFeaturesProp
  
  groupNameProp = cmd.createProperty(VC_STRING, 'Group Name')
  
  setGroupProp = cmd.createProperty(VC_BUTTON, 'Set Color Group')
  setGroupProp.OnChanged = SetGroup
  
  colorGroupsProp = cmd.createProperty(VC_STRING, 'Color Groups', VC_PROPERTY_STEP)
  colorGroupsProp.OnChanged = OnColorGroupChanged
  colorGroupsProp.IsVisible = False
  
  colorProp = cmd.createProperty('Ref<Material>', 'Color')
  colorProp.OnChanged = OnColorChanged
  colorProp.IsVisible = False
  
  removeGroupProp = cmd.createProperty(VC_BUTTON, 'Remove Color Group')
  removeGroupProp.OnChanged = RemoveGroup
  removeGroupProp.IsVisible = False
  
  selectFeaturesProp = cmd.createProperty(VC_BUTTON, 'Select Group Features')
  selectFeaturesProp.OnChanged = SelectGroupFeatures
  selectFeaturesProp.IsVisible = False
  
  selectionManager.OnSelectionChanged = UpdateUI
  
  executeInActionPanel()
  
  UpdateUI()


app = getApplication()
cmd = getCommand()

selectionManager = app.SelectionManager

propPrefix = 'Colors::'
scriptName = 'ColorGroupsScript'

suppressColorChange = False

updateColorsScript = """
from vcScript import *

def OnColorChanged(arg):
  featureIds = comp.getProperty(arg.Name + '_').Value.split(',')

  for featureId in featureIds:
    feature = comp.findFeature(featureId)

    if feature is None:
      print 'WARNING: Could not find feature with id \"' + featureId + '\"'
    else:
      UpdateFeatureMaterials(feature, arg.Value)
  
  comp.rebuild()
  app.render()

def UpdateFeatureMaterials(feature, material):
  if len(feature.Children) > 0:
    for child in feature.Children:
      UpdateFeatureMaterials(child, material)
  else:
    print 'Coloring feature with id \"' + feature.Name + '\"...'

    if hasattr(feature, 'Material'):
      feature.Material = material
    elif hasattr(feature, 'Geometry'):
      geometrySets = feature.Geometry.GeometrySets

      for geometrySet in geometrySets:
        geometrySet.Material = material

app = getApplication()
comp = getComponent()

colorProps = [x for x in comp.Properties
  if x.Name.startswith('""" + propPrefix + """') and x.Name[-1] != '_']

for i in range(len(colorProps)):
  colorProps[i].OnChanged = OnColorChanged"""

addState(None)